==> apracjs-13.1.29.tgz: package/lib/scripts/dataSource/pagerInfo.js <==
﻿/*!
 * PagerInfoData
 * Copyright(c) 2017 by wssf
 * 2017-06-13
 * 处理数据库分页数据源
 */

'use strict'
var debug = require('debug')('aprac:gspxCache');
var utils = require('../../common/utils.js');
var config = require('../../config.js').apracConfig;
var sysConsts = require('../sysConsts.json');
var cacheManager = require('../../common/cacheManager.js');

class PagerInfoData {
  constructor(session, pagerId, dataSource, src) {
    this._cacheId = session.id + pagerId;
    this._createdCount = 1;
    this._oldQueryParams = null;
    this._oldItemCount = 0;
    this._src = src;
    this._isFirstLoad = true;
    cacheManager.add(this._cacheId, new PagerInfo(session, pagerId, dataSource));
  }
}

class PagerInfo {
  constructor(session, pagerId, dataSource) {
    if (!pagerId)
      throw new TypeError("pagerId");
    if (!dataSource)
      throw new TypeError("dataSource");

    dataSource._sessionID = session.id;
    dataSource._pagerId = pagerId;

    this._dataSource = dataSource;
  }

  getDataSource() {
    return this._dataSource;
  }

  getPagerData(queryParams, orders, filter, first, count, items) {
    if (filter === 'null') filter = null;
    if (orders === 'null') orders = null;
    if (items === 'null') items = null;
    if (queryParams === 'null') queryParams = null;

    var req = this.req;
    var dataSource = this.getDataSource();
    if (req != null) {
      var isReporting = req.headers['user-agent'].toLowerCase().indexOf("indy library") >= 0;
      if (queryParams != null && queryParams["__IsReporting"]) {
        isReporting = true;
        queryParams = null;
      }
      dataSource._isReporting = isReporting;
    }
    var needQuery = queryParams != null;
    if (needQuery) {
      dataSource._realRowItemCount = false;
      dataSource.setQueryParams(queryParams);
    }
    if (items != null) {
      dataSource.invalidateCache();
      needQuery = true;
    }
    if (orders != null && typeof dataSource.setOrders == "function") {
      dataSource.setOrders(orders);
    }
    if (typeof dataSource.setFilter == "function") {
      dataSource.setFilter(filter);
    }

    var hasColumnExpand = needQuery;
    var answer = this.getPagerDataByIndex(first, count, hasColumnExpand, needQuery);

    if (dataSource._isReporting && needQuery) // 打印全部时会重传查询参数导致重新查询 bug4844
    {
      // to do
    }
    dataSource._realRowItemCount = true;
    return answer;
  }

  getPagerSummary() {
    var req = this.req;
    var dataSource = this.getDataSource();

    var items = {};
    if (typeof dataSource.addSummaryItems == "function") {
      dataSource.addSummaryItems(items);
    }

    if (typeof dataSource.calcSummaryItems == "function") {
      dataSource.calcSummaryItems(items);
    }

    return items;
  }

  //方法包
  wrapInterceptor(interceptor, funname, obj, args) {
    return async (cb) => {
      try {
        if (interceptor) {
          await interceptor.before(obj, funname, args);
        }
        let rt = await obj[funname].apply(obj, args);
        await rt(async (result, errormsg) => {
          if (interceptor) {
            await interceptor.after(obj, funname, args, result, errormsg);
          }
          cb(result, errormsg);
        })
      } catch (e) {
        cb(null, e);
      }
    }
  }

  getPagerDataByIndex(first, count, hasColumnExpand, needQuery) {
    var itemList;
    var itemCount;
    var dataSource = this.getDataSource();
    var interceptor = dataSource && dataSource.Interceptor ? dataSource.Interceptor : null;
    if (count >= 0) {
      itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [first, count]); //dataSource.getItemList(first, count);
      itemCount = dataSource.getItemCount();
    } else {
      if (dataSource) {
        itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [0, -1]); //dataSource.getItemList(0, -1);
        itemCount = dataSource.getItemCount();
      } else {
        itemCount = dataSource.getItemCount(); // 这时只能先取合计
        itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [0, itemCount]); //dataSource.getItemList(0, itemCount);
      }
    }
    var pagerData = {};
    pagerData["itemList"] = itemList;
    pagerData["itemCount"] = itemCount;
    if (hasColumnExpand) {
      var value = PagerInfo.getDataSourceColumnExpands(dataSource);
      if (value != null) {
        pagerData["columnExpands"] = value;
      }
    }
    if (typeof dataSource.addCellMergeItems == "function") {
      pagerData["cellMergeItems"] = this.getDataSourceCellMergeItems(dataSource);
    }
    if (needQuery && typeof dataSource.getQueryState == "function") // 重新查询时才重新取
    {
      pagerData["queryState"] = dataSource.getQueryState();
    }

    return pagerData;
  }

  getDataSourceCellMergeItems(dataSource) {
    return null;
  }
}

PagerInfo.getDataSourceColumnExpands = function (dataSource) {
  if (typeof (dataSource.expandColumn) == 'function') {
    var info = {
      name: '',
      captions: [],
      dataFields: [],
      reportFields: []
    };
    dataSource.expandColumn(info);

    if (info.name) {
      return [info];
    }
  } else if (typeof (dataSource.expandColumns) == 'function') {
    var infoList = [];
    dataSource.expandColumns(infoList);
    if (infoList.length > 0) {
      return infoList;
    }
  }
  return null;
}

exports = module.exports = PagerInfo;
exports.PagerInfoData = PagerInfoData;

==> apracjs-18.1.2.tgz: package/lib/scripts/dataSource/pagerInfo.js <==
﻿/*!
 * PagerInfoData
 * Copyright(c) 2017 by wssf
 * 2017-06-13
 * 处理数据库分页数据源
 */

'use strict'
var debug = require('debug')('aprac:gspxCache');
var utils = require('../../common/utils.js');
var config = require('../../config.js').apracConfig;
var sysConsts = require('../sysConsts.json');
var cacheManager = require('../../common/cacheManager.js');

class PagerInfoData {
  constructor(session, pagerId, dataSource, src) {
    this._cacheId = session.id + pagerId;
    this._createdCount = 1;
    this._oldQueryParams = null;
    this._oldItemCount = 0;
    this._src = src;
    this._isFirstLoad = true;
    cacheManager.add(this._cacheId, new PagerInfo(session, pagerId, dataSource), session.id);
  }
}

class PagerInfo {
  constructor(session, pagerId, dataSource) {
    if (!pagerId)
      throw new TypeError("pagerId");
    if (!dataSource)
      throw new TypeError("dataSource");

    dataSource._sessionID = session.id;
    dataSource._pagerId = pagerId;

    this._dataSource = dataSource;
  }

  dispose() {
    if (this._dataSource) {
      this._dataSource.invalidateCache();
      this._dataSource = null;
    }
  }

  getDataSource() {
    return this._dataSource;
  }

  getPagerData(queryParams, orders, filter, first, count, items) {
    if (filter === 'null') filter = null;
    if (orders === 'null') orders = null;
    if (items === 'null') items = null;
    if (queryParams === 'null') queryParams = null;

    var req = this.req;
    var dataSource = this.getDataSource();
    if (req != null) {
      var isReporting = req.headers['user-agent'].toLowerCase().indexOf("indy library") >= 0;
      if (queryParams != null && queryParams["__IsReporting"]) {
        isReporting = true;
        queryParams = null;
      }
      dataSource._isReporting = isReporting;
    }
    var needQuery = queryParams != null;
    if (needQuery) {
      dataSource._realRowItemCount = false;
      dataSource.setQueryParams(queryParams);
    }
    if (items != null) {
      dataSource.invalidateCache();
      needQuery = true;
    }
    if (orders != null && typeof dataSource.setOrders == "function") {
      dataSource.setOrders(orders);
    }
    if (typeof dataSource.setFilter == "function") {
      dataSource.setFilter(filter);
    }

    var hasColumnExpand = needQuery;
    var answer = this.getPagerDataByIndex(first, count, hasColumnExpand, needQuery);

    if (dataSource._isReporting && needQuery) // 打印全部时会重传查询参数导致重新查询 bug4844
    {
      // to do
    }
    dataSource._realRowItemCount = true;
    return answer;
  }

  getPagerSummary() {
    var req = this.req;
    var dataSource = this.getDataSource();

    var items = {};
    if (typeof dataSource.addSummaryItems == "function") {
      dataSource.addSummaryItems(items);
    }

    if (typeof dataSource.calcSummaryItems == "function") {
      dataSource.calcSummaryItems(items);
    }

    return items;
  }

  //方法包
  wrapInterceptor(interceptor, funname, obj, args) {
    return async (cb) => {
      try {
        if (interceptor) {
          await interceptor.before(obj, funname, args);
        }
        let rt = await obj[funname].apply(obj, args);
        await rt(async (result, errormsg) => {
          if (interceptor) {
            await interceptor.after(obj, funname, args, result, errormsg);
          }
          cb(result, errormsg);
        })
      } catch (e) {
        cb(null, e);
      }
    }
  }

  getPagerDataByIndex(first, count, hasColumnExpand, needQuery) {
    var itemList;
    var itemCount;
    var dataSource = this.getDataSource();
    var interceptor = dataSource && dataSource.Interceptor ? dataSource.Interceptor : null;
    if (count >= 0) {
      itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [first, count]); //dataSource.getItemList(first, count);
      itemCount = dataSource.getItemCount();
    } else {
      if (dataSource) {
        itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [0, -1]); //dataSource.getItemList(0, -1);
        itemCount = dataSource.getItemCount();
      } else {
        itemCount = dataSource.getItemCount(); // 这时只能先取合计
        itemList = this.wrapInterceptor(interceptor, "getItemList", dataSource, [0, itemCount]); //dataSource.getItemList(0, itemCount);
      }
    }
    var pagerData = {};
    pagerData["itemList"] = itemList;
    pagerData["itemCount"] = itemCount;
    if (hasColumnExpand) {
      var value = PagerInfo.getDataSourceColumnExpands(dataSource);
      if (value != null) {
        pagerData["columnExpands"] = value;
      }
    }
    if (typeof dataSource.addCellMergeItems == "function") {
      pagerData["cellMergeItems"] = this.getDataSourceCellMergeItems(dataSource);
    }
    if (needQuery && typeof dataSource.getQueryState == "function") // 重新查询时才重新取
    {
      pagerData["queryState"] = dataSource.getQueryState();
    }

    return pagerData;
  }

  getDataSourceCellMergeItems(dataSource) {
    return null;
  }
}

PagerInfo.getDataSourceColumnExpands = function(dataSource) {
  if (typeof (dataSource.expandColumn) == 'function') {
    var info = {
      name: '',
      captions: [],
      dataFields: [],
      reportFields: []
    };
    dataSource.expandColumn(info);

    if (info.name) {
      return [info];
    }
  } else if (typeof (dataSource.expandColumns) == 'function') {
    var infoList = [];
    dataSource.expandColumns(infoList);
    if (infoList.length > 0) {
      return infoList;
    }
  }
  return null;
}

exports = module.exports = PagerInfo;
exports.PagerInfoData = PagerInfoData;
